// -*- C++ -*-
/*!
 * @file  WiiNunchakuTranslator.cpp
 * @brief WiiNunchakuTranslator
 * @date $Date$
 *
 * @author 〒135-8548
 * 東京都江東区豊洲3-7-5　芝浦工業大学　研究棟　11-Q-32
 * 水川(HRI)研究室
 * 代表者:水川　真   (Emai:mizukawa<at>sic.shibaura-it.ac.jp)
 * 作成者;田中　基雅(Email:m108081<at>shibaura-it.ac.jp)
 *          藤田　恒彦(Email:m109077<at>shibaura-it.ac.jp)
 *          鷹栖　尭大（Email:m109056<at>shibaura-it.ac.jp）
 *
 * WiiNunchakuTranslatorについての著作権は，芝浦工業大学 水川研究室に帰属します．
 * 非営利目的利用は自由です．
 *
 * $Id$
 */

#include "WiiNunchakuTranslator.h"

// Module specification
// <rtc-template block="module_spec">
static const char* wiinunchakutranslator_spec[] =
  {
    "implementation_id", "WiiNunchakuTranslator",
    "type_name",         "WiiNunchakuTranslator",
    "description",       "WiiNunchakuTranslator",
    "version",           "1.0.0",
    "vendor",            "Shibaura Institute of Technology",
    "category",          "RemoteController",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.debug", "0",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
WiiNunchakuTranslator::WiiNunchakuTranslator(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_WiiDataIn("WiiData", m_WiiData),
    m_WiiConvertOut("WiiConvert", m_WiiConvert),
    m_WiiInfoPort("WiiInfo")

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
WiiNunchakuTranslator::~WiiNunchakuTranslator()
{
}


/*!
 * Configurationの初期化を行う
 */
RTC::ReturnCode_t WiiNunchakuTranslator::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("WiiData", m_WiiDataIn);
  
  // Set OutPort buffer
  registerOutPort("WiiConvert", m_WiiConvertOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_WiiInfoPort.registerConsumer("WiiInfo", "WiiInfo", m_WiiInfo);
  
  // Set CORBA Service Ports
  registerPort(m_WiiInfoPort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("debug", m_debug, "0");
  
  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t WiiNunchakuTranslator::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiNunchakuTranslator::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiNunchakuTranslator::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * キャリブレーションデータを取得し，WiiRemoteAgentに送信する．
 */

RTC::ReturnCode_t WiiNunchakuTranslator::onActivated(RTC::UniqueId ec_id)
{
	//キャリブレーションデータ格納用
	Calibration_var CalibList;
	BYTE status = 0x00;
	BYTE buff[255];

	while(1)
	{
		CalibList = m_WiiInfo->get_wii_mote_calib();
		status = CalibList[0];
		if(status == 0x21)
		{
			break;
		}
		Sleep(50);
	}
	
	for(int i = 0; i < 22 ; i++){
		buff[i] = CalibList[i];
	}
	wii.SetWiiRemoteCalibration(buff);

	status = 0x00;
	while(1)
	{
		CalibList = m_WiiInfo->get_wii_ext_calib();
		status = CalibList[0];
		if(status == 0x21)
		{
			break;
		}
		Sleep(50);
	}

	for(int i = 0; i < 22 ; i++){
		buff[i] = CalibList[i];
	}
	wii.SetWiiExtensionCalibration(buff);

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t WiiNunchakuTranslator::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * WiiRemoteAgentから生データが入力された時に，それを整形データに変換し，出力する
 * ．
 * 整形データはTimedShortSeq型で出力する．
 * データの内訳を以下に記す．
 * 尚，[0]から[13]までがWiiリモコン本体の情報，それ以降はNunchakuの情報となっている
 * ．
 * [0] 十字キー・上ボタンの状態(BOOL 値)
 * [1] 十字キー・右ボタンの状態(BOOL 値)
 * [2] 十字キー・下ボタンの状態(BOOL 値)
 * [3] 十字キー・左ボタンの状態(BOOL 値)
 * [4] A ボタンの状態(BOOL 値)
 * [5] B ボタンの状態(BOOL 値)
 * [6] ＋ボタンの状態(BOOL 値)
 * [7] Home ボタンの状態(BOOL 値)
 * [8] －ボタンの状態(BOOL 値)
 * [9] 1 ボタンの状態(BOOL 値)
 * [10] 2 ボタンの状態(BOOL 値)
 * [11] X軸方向の加速度
 * [12] Y軸方向の加速度
 * [13] Z軸方向の加速度
 * [14] C ボタンの状態(BOOL 値)
 * [15] Z ボタンの状態(BOOL 値)
 * [16] アナログスティックのX 方向入力値
 * (解放時0，最大まで左に傾けると-100，同右で100)
 * [17] アナログスティックのY 方向入力値
 * (解放時0，最大まで左に傾けると-100，同右で100)
 * [18] X軸方向の加速度
 * [19] Y軸方向の加速度
 * [20] Z軸方向の加速度
 */

RTC::ReturnCode_t WiiNunchakuTranslator::onExecute(RTC::UniqueId ec_id)
{
	BYTE data[22];
	if(m_WiiDataIn.isNew())
	{
		m_WiiDataIn.read();
		for(int i = 0 ; i < 22 ; i++)
		{
			data[i] = m_WiiData.data[i];	
		}

		wii.ParseButtons(data);
		wii.ParseAccel(data);
		wii.ParseExtension(data);

		m_WiiConvert.data.length(21);
		m_WiiConvert.data[0] = wii.mWiiRemoteState.ButtonState.Up;
		m_WiiConvert.data[1] = wii.mWiiRemoteState.ButtonState.Right;
		m_WiiConvert.data[2] = wii.mWiiRemoteState.ButtonState.Down;
		m_WiiConvert.data[3] = wii.mWiiRemoteState.ButtonState.Left;
		m_WiiConvert.data[4] = wii.mWiiRemoteState.ButtonState.A;
		m_WiiConvert.data[5] = wii.mWiiRemoteState.ButtonState.B;
		m_WiiConvert.data[6] = wii.mWiiRemoteState.ButtonState.Plus;
		m_WiiConvert.data[7] = wii.mWiiRemoteState.ButtonState.Home;
		m_WiiConvert.data[8] = wii.mWiiRemoteState.ButtonState.Minus;
		m_WiiConvert.data[9] = wii.mWiiRemoteState.ButtonState.One;
		m_WiiConvert.data[10] = wii.mWiiRemoteState.ButtonState.Two;
		m_WiiConvert.data[11] = wii.mWiiRemoteState.AccelState.Values.X;
		m_WiiConvert.data[12] = wii.mWiiRemoteState.AccelState.Values.Y;
		m_WiiConvert.data[13] = wii.mWiiRemoteState.AccelState.Values.Z;
		m_WiiConvert.data[14] = wii.NunchukState.C;
		m_WiiConvert.data[15] = wii.NunchukState.Z;
		m_WiiConvert.data[16] = wii.NunchukState.Joystick.X;
		m_WiiConvert.data[17] = wii.NunchukState.Joystick.Y;
		m_WiiConvert.data[18] = wii.NunchukState.AccelState.Values.X;
		m_WiiConvert.data[19] = wii.NunchukState.AccelState.Values.Y;
		m_WiiConvert.data[20] = wii.NunchukState.AccelState.Values.Z;
		

		if(m_debug == true)
		{
			system("cls");
			std::cout << "<<WiiRemoteState>>" << std::endl;
			std::cout << "↑:" << m_WiiConvert.data[0] << std::endl;
			std::cout << "→:" << m_WiiConvert.data[1] << std::endl;
			std::cout << "↓:" << m_WiiConvert.data[2] << std::endl;
			std::cout << "←:" << m_WiiConvert.data[3] << std::endl;
			std::cout << "Ａ:" << m_WiiConvert.data[4] << std::endl;
			std::cout << "Ｂ:" << m_WiiConvert.data[5] << std::endl;
			std::cout << "＋:" << m_WiiConvert.data[6] << std::endl;
			std::cout << "Home:" << m_WiiConvert.data[7] << std::endl;
			std::cout << "－:" << m_WiiConvert.data[8] << std::endl;
			std::cout << "１:" << m_WiiConvert.data[9] << std::endl;
			std::cout << "２:" << m_WiiConvert.data[10] << std::endl;
			std::cout << "X_Accel:" << m_WiiConvert.data[11] << std::endl;
			std::cout << "Y_Accel:" << m_WiiConvert.data[12] << std::endl;
			std::cout << "Z_Accel:" << m_WiiConvert.data[13] << std::endl;
			std::cout << std::endl;

			std::cout << "<<WiiNunchakuState>>" << std::endl;
			std::cout << "Ｃ:" << m_WiiConvert.data[14] << std::endl;
			std::cout << "Ｚ:" << m_WiiConvert.data[15] << std::endl;
			std::cout << "X_Joystick:" << m_WiiConvert.data[16] << std::endl;
			std::cout << "Y_Joystick:" << m_WiiConvert.data[17] << std::endl;
			std::cout << "X_Accel:" << m_WiiConvert.data[18] << std::endl;
			std::cout << "Y_Accel:" << m_WiiConvert.data[19] << std::endl;
			std::cout << "Z_Accel:" << m_WiiConvert.data[20] << std::endl;
		}

		m_WiiConvertOut.write();

	}
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t WiiNunchakuTranslator::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiNunchakuTranslator::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiNunchakuTranslator::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiNunchakuTranslator::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiNunchakuTranslator::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void WiiNunchakuTranslatorInit(RTC::Manager* manager)
  {
    coil::Properties profile(wiinunchakutranslator_spec);
    manager->registerFactory(profile,
                             RTC::Create<WiiNunchakuTranslator>,
                             RTC::Delete<WiiNunchakuTranslator>);
  }
  
};


