#include "WiiRemoteCntl.h"
#include "WiiBalanceBoard.h"

WiiBalanceBoard::WiiBalanceBoard(void)
{
	isExtension = false;
	LEDandVibrationCommand = 0x00;
}

WiiBalanceBoard::~WiiBalanceBoard(void)
{
}

bool WiiBalanceBoard::Initialize(void)
{
	if(!FindWiiRemote())
	{
		std::cerr << "not found wii remote" <<std::endl;
		return false;
	}

	Report_0x12(0x00,0x31);
	GetStatus();
	isStartThread = true;
	hThread = (HANDLE)_beginthreadex(NULL, 0, this->OnReceive, this, 0, NULL);
	Sleep(500);
	return true;
}


void WiiBalanceBoard::InitializeExtension()
{
	BYTE *buff = ReadData(0x04a400fa, 6);
	if(!((wiiExtState[9]==buff[9])&(wiiExtState[10]==buff[10])&(wiiExtState[11]==buff[11])))
	{
		WriteData(0x04a400f0,0x55);
		Sleep(50);
		WriteData(0x04a400fb,0x00);
		Sleep(50);
	}

	buff = ReadData(0x04a400fa, 6);
	memcpy(wiiExtState,buff,22);
	long type = ((long)buff[9]) << 16 | ((long)buff[10]) << 8 | buff[11];

	ZeroMemory(&wiiRemoteExtensionCalibrationData, sizeof(wiiRemoteExtensionCalibrationData));

	if(type == 0x200402)
	{
		buff = ReadData(0x04a40020, 32);
		memcpy(wiiRemoteExtensionCalibrationData,buff,44);
		Report_0x12(0x00,0x34);
	}
	else
	{
		std::cout << "oXȊOڑĂ" << std::endl;
	}

	Sleep(50);
	return;
}

bool WiiBalanceBoard::WiiRemoteThread()
{	
	Sleep(1);
	BYTE* buff = ReadReport();

	switch(buff[0])
	{
	case 0x20: // Xe[^X
		{
			// g
			isExtension = (buff[3] & 0x02) != 0;
			memcpy(wiiStateData,buff,22);
			if(isExtension)
			{
				InitializeExtension();
			}
			else
			{
				Report_0x12(0x04,0x37);
			}
			break;
		}
	default:
		memcpy(mReadBuff,buff,22);	
		break;
	}
	return true;
}

bool WiiBalanceBoard::Finalize(void)
{
	CloseThread();
	return WiiClosed();
}

void WiiBalanceBoard::CloseThread()
{
	if(isStartThread == true)
	{
		if (hThread != NULL) 
		{
			isStartThread = false;
			//Xbh̏I҂
			WaitForSingleObject(hThread, INFINITE);

			hThread = NULL;
			CloseHandle(hThread);
		}
	}
}

BYTE *WiiBalanceBoard::ReadData(int address, int size)
{
	BYTE outputReport[22];
	BYTE readBuff[256];
	int counter = 0;
	int dataCount = 0;
	bool isResetEvent = false;

	// CXbhU~
	if(hThread)
	{
		isResetEvent = true;
		ResetEvent(hThread);
	}
	
	ZeroMemory(&outputReport, sizeof(outputReport));
	ZeroMemory(&readBuff, sizeof(readBuff));
	ZeroMemory(&mReadBuff_17, sizeof(mReadBuff_17));

	outputReport[0] = 0x17;
	outputReport[1] = ((address & 0xff000000) >> 24);
	outputReport[2] = ((address & 0x00ff0000) >> 16);
	outputReport[3] = ((address & 0x0000ff00) >>  8);
	outputReport[4] = (address & 0x000000ff);
	outputReport[5] = (size & 0xff00) >> 8;
	outputReport[6] = size & 0xff;
	WriteReport(outputReport);

	while(1)
	{
		ReadReport(readBuff,&inputLength);
		if(counter > 100)
		{
			std::cout << "f[^̎MɎs܂D" << std::endl;
			break;
		}

		int buffsize = (readBuff[3] >> 4) + 1;
		int offset = (readBuff[4] << 8 | readBuff[5]);

		if(readBuff[0] == 0x21)
		{
			if(dataCount * 22 > 255)
			{
				std::cout << "obt@TCY܂" << std::endl;
				break;
			}
			memcpy(mReadBuff_17 + dataCount * 22,readBuff,inputLength);
			dataCount++;
			if(((address & 0xffff) + size) == (buffsize + offset))
			{
				break;
			}
		}
		counter++;
	}
	
	if(isResetEvent)
	{
		SetEvent(hThread);
	}

	return mReadBuff_17;
}

unsigned __stdcall WiiBalanceBoard::OnReceive(void* myself)
{
	WiiBalanceBoard &wiiNu = *(WiiBalanceBoard*)myself;
	while(1)
	{	
		wiiNu.WiiRemoteThread();
		if(wiiNu.isStartThread == false)
		{
			break;
		}
	}
	return 0;
}

bool WiiBalanceBoard::SetLED_Vibration(BYTE setCommand)
{
	bool isResetEvent = false;
	LEDandVibrationCommand = setCommand;

	// CXbhU~
	if(hThread)
	{
		isResetEvent = true;
		ResetEvent(hThread);
	}

	BYTE command[2];
	command[0] = 0x11;
	command[1] = setCommand;

	WriteReport(command);

	if(isResetEvent)
	{
		SetEvent(hThread);
	}

	return true;
}

bool WiiBalanceBoard::RequestCalib(BYTE *input)
{
	memcpy(input,wiiRemoteCalibrationData,22);
	return true;
}

bool WiiBalanceBoard::RequestCalibExt(BYTE *input)
{
	memcpy(input,wiiRemoteExtensionCalibrationData,44);
	return true;
}

bool WiiBalanceBoard::RequestData(BYTE* input)
{
	memcpy(input,mReadBuff,22);
	return true;
}

bool WiiBalanceBoard::RequestState(BYTE* input)
{
	GetStatus();
	Sleep(15);
	//obt@܂Ă邱Ƃ\z邽߁C_ōXV
	memcpy(input, wiiStateData,22);
	return true;
}

